/**
 * A checkbox field
 */

import Const from 'uxcore-const';
import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames';
import TableContext from '../context';

class CheckBox extends React.Component {
  constructor(props) {
    super(props);
    this.handleClick = this.handleClick.bind(this);
  }

  getValue() {
    return this.checkbox.checked;
  }

  handleChange(e) {
    this.props.onChange(e);
  }

  handleClick() {
    // simulate checkbox
    this.props.onChange({
      target: {
        checked: true,
      },
    });
  }

  renderHalfChecked() {
    return (
      <TableContext.Consumer>
        {context => (
          <span
            className={classnames({
              [`${context.prefixCls}-row-selector`]: true,
              [this.props.className]: !!this.props.className,
            })}
          >
            <i className="half-checked" onClick={this.handleClick} />
          </span>
        )}
      </TableContext.Consumer>
    );
  }

  renderCheckBox(renderProps) {
    return (
      <TableContext.Consumer>
        {context => (
          <label
            className={classnames({
              [`${context.prefixCls}-row-selector`]: true,
              [this.props.className]: !!this.props.className,
            })}
          >
            <input type="checkbox" ref={(c) => { this.checkbox = c; }} {...renderProps} />
            <s />
            {' ' + renderProps.text || null}
          </label>
        )}
      </TableContext.Consumer>
    );
  }

  render() {
    const { props } = this;
    if (props.halfChecked) {
      return this.renderHalfChecked();
    }
    const renderProps = {
      className: 'kuma-checkbox',
      checked: this.props.checked || false,
      onChange: this.handleChange.bind(this),
      text: this.props.text
    };
    if (!!props.disable || props.mode === Const.MODE.VIEW) {
      renderProps.disabled = true;
    }
    return this.renderCheckBox(renderProps);
  }
}


CheckBox.propTypes = {
  onChange: PropTypes.func,
  checked: PropTypes.bool,
  className: PropTypes.string,
  text: PropTypes.string
};

CheckBox.defaultProps = {
  onChange: () => {},
  checked: undefined,
  className: '',
  text: ''
};

export default CheckBox;
